@@ -4,6 +4,7 @@
 --   have been in base, or could plausibly be added.
 module General.Base(module General.Base, module X) where
 
+import Control.Applicative as X ((<*>),(<$>))
 import Control.Arrow as X
 import Control.DeepSeq as X
 import Control.Monad as X
@@ -20,7 +21,6 @@ import Data.Int as X
 import Data.Word as X
 import Debug.Trace as X (trace)
 import Numeric as X (readHex,showHex)
-import System.FilePath as X hiding (combine)
 import qualified Data.ByteString.Char8 as BS
 import qualified Data.ByteString.Lazy.Char8 as LBS
 
@@ -109,6 +109,23 @@ readFileUtf8 x = do
 #endif
 
 
+readFileLatin1' :: FilePath -> IO String
+readFileLatin1' x = do
+    src <- readFileLatin1 x
+    length src `seq` return src
+
+
+readFileLatin1 :: FilePath -> IO String
+#if __GLASGOW_HASKELL__ < 612
+readFileLatin1 x = readFile x
+#else
+readFileLatin1 x = do
+    h <- openFile x ReadMode
+    hSetEncoding h latin1
+    hGetContents h
+#endif
+
+
 writeFileUtf8 :: FilePath -> String -> IO ()
 #if __GLASGOW_HASKELL__ < 612
 writeFileUtf8 x y = writeFile x y
