@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE PatternGuards #-}
 
 {- |
@@ -12,16 +13,30 @@ module General.Web(
     Args, cgiArgs, cgiResponse, parseHttpQueryArgs
     ) where
 
+#ifndef MIN_VERSION_wai
+#define MIN_VERSION_wai(a,b,c) 0
+#endif
+
 import General.System
 import General.Base
+import System.FilePath
 import Network.Wai
+#if MIN_VERSION_wai(3, 0, 0)
+import Data.IORef
+#endif
+#if MIN_VERSION_wai(2, 0, 0)
+import Network.Wai.Internal
+#endif
 import Network.HTTP.Types
 import Data.CaseInsensitive(original)
 import qualified Data.ByteString.Lazy.Char8 as LBS
 
 import Blaze.ByteString.Builder(toLazyByteString)
 import Data.Conduit.List(consume)
-import Data.Conduit(($$),Flush,runResourceT,Flush(Chunk))
+import Data.Conduit(($$),Flush(Chunk))
+#if !MIN_VERSION_wai(2, 0, 0)
+import Control.Monad.Trans.Resource (runResourceT)
+#endif
 
 type Args = [(String, String)]
 
@@ -34,10 +49,25 @@ responseNotFound x = responseLBS status404 [] $ fromString $ "File not found: "
 
 responseFlatten :: Response -> IO (Status, ResponseHeaders, LBString)
 responseFlatten r = do
+#if MIN_VERSION_wai(3, 0, 0)
+    let (s,hs,withBody) = responseToStream r
+    ref <- newIORef mempty
+    let addChunk builder = modifyIORef ref (<> builder)
+    withBody $ \body -> body addChunk (return ())
+    builder <- readIORef ref
+    let res = toLazyByteString builder
+    return (s,hs,res)
+#elif MIN_VERSION_wai(2, 0, 0)
+    let (s,hs,withSrc) = responseToSource r
+    chunks <- withSrc $ \src -> src $$ consume
+    let res = toLazyByteString $ mconcat [x | Chunk x <- chunks]
+    return (s,hs,res)
+#else
     let (s,hs,rest) = responseSource r
     chunks <- runResourceT $ rest $$ consume
     let res = toLazyByteString $ mconcat [x | Chunk x <- chunks]
     return (s,hs,res)
+#endif
 
 
 responseEvaluate :: Response -> IO ()
